@@ -3,12 +3,15 @@
 This module contains classes associated with manipulating vector data.
 """
 import geopandas
+import pandas
 import shapely
 import numpy
 import pathlib
 import typing
 import logging
 
+RASTER_TYPE = numpy.float32
+
 
 class CatchmentGeometry:
     """A class defining revelant catchment regions by polygons.
@@ -52,6 +55,7 @@ class CatchmentGeometry:
 
         # values set in setup once land has been specified
         self._land = None
+        self._full_land = None
         self._foreshore = None
         self._land_and_foreshore = None
         self._foreshore_and_offshore = None
@@ -61,10 +65,10 @@ class CatchmentGeometry:
         """Define the main catchment regions and ensure the CRS is set for
         each region"""
 
-        self._land = self._land.to_crs(self.crs["horizontal"])
+        self._full_land = self._full_land.to_crs(self.crs["horizontal"])
 
         # Clip and remove any sub pixel regions
-        self._land = self._catchment.clip(self._land, keep_geom_type=True)
+        self._land = self._catchment.clip(self._full_land, keep_geom_type=True)
         self._land = self._land[self._land.area > self.resolution * self.resolution]
 
         self._foreshore_and_offshore = self.catchment.overlay(
@@ -83,7 +87,7 @@ class CatchmentGeometry:
             self._land_and_foreshore.area > self.resolution * self.resolution
         ]
 
-        self._foreshore = self.land_and_foreshore.overlay(self.land, how="difference")
+        self._foreshore = self._land_and_foreshore.overlay(self.land, how="difference")
 
         self._offshore = self.catchment.overlay(
             self.land_and_foreshore, how="difference"
@@ -114,11 +118,18 @@ class CatchmentGeometry:
         self._assert_land_set()
         return self._land
 
+    @property
+    def full_land(self):
+        """Return the full land region"""
+
+        self._assert_land_set()
+        return self._full_land
+
     @land.setter
     def land(self, land_file: typing.Union[str, pathlib.Path]):
         """Set the land region and finish setup."""
 
-        self._land = geopandas.read_file(land_file)
+        self._full_land = geopandas.read_file(land_file)
 
         self._set_up()
 
@@ -218,7 +229,7 @@ class CatchmentGeometry:
         offshore_foreshore_dense_data_extents = geopandas.GeoDataFrame(
             {
                 "geometry": [
-                    shapely.ops.cascaded_union(
+                    shapely.ops.unary_union(
                         [self.foreshore.loc[0].geometry, dense_extents.loc[0].geometry]
                     )
                 ]
@@ -303,20 +314,24 @@ class BathymetryContours:
 
         self._contour = self._contour.to_crs(self.catchment_geometry.crs["horizontal"])
 
+        # Define a large offshore area to keep ocean contours in - to ensure correct
+        # interpolation behaviour near the edges
+        offshore = self.catchment_geometry.offshore
+        offshore = geopandas.GeoDataFrame(
+            geometry=offshore.buffer(numpy.sqrt(offshore.area.sum()))
+        )
+        offshore = offshore.overlay(self.catchment_geometry.land, how="difference")
+
         if exclusion_extent is not None:
             # Remove areas already covered by LiDAR - drop any polygons less than a
             # pixel in area
-            exclusion_extent = exclusion_extent.clip(
-                self.catchment_geometry.offshore, keep_geom_type=True
-            )
+            exclusion_extent = exclusion_extent.clip(offshore, keep_geom_type=True)
             exclusion_extent = exclusion_extent[
                 exclusion_extent.area > self.catchment_geometry.resolution**2
             ]
-            self._extent = self.catchment_geometry.offshore.overlay(
-                exclusion_extent, how="difference"
-            )
+            self._extent = offshore.overlay(exclusion_extent, how="difference")
         else:
-            self._extent = self.catchment_geometry.offshore
+            self._extent = offshore
         # Keep only contours in the 'extents' i.e. inside the catchment and outside any
         # exclusion_extent
         self._contour = self._contour.clip(self._extent, keep_geom_type=True)
@@ -359,7 +374,7 @@ class BathymetryContours:
 
         points = numpy.empty(
             [points_df.apply(lambda row: len(row.geoms)).sum()],
-            dtype=[("X", numpy.float64), ("Y", numpy.float64), ("Z", numpy.float64)],
+            dtype=[("X", RASTER_TYPE), ("Y", RASTER_TYPE), ("Z", RASTER_TYPE)],
         )
 
         # Extract the x, y and z values from the Shapely MultiPoints and possibly a
@@ -474,8 +489,10 @@ class EstimatedBathymetryPoints:
     used to interpolate elevations within the river and mouth polygon.
     """
 
-    DEPTH_LABEL = "depths"
+    DEPTH_LABEL = "elevation"
     TYPE_LABEL = "type"
+    BANK_HEIGHT_LABEL = "bank_height"
+    WIDTH_LABEL = "width"
 
     def __init__(
         self,
@@ -512,25 +529,47 @@ class EstimatedBathymetryPoints:
         ), "All bathy points should have a type label.Instead there are "
         f"{len(points_files)} points files and {len(type_labels)} type labels"
 
-        points = geopandas.read_file(points_files[0])
-        points[self.TYPE_LABEL] = type_labels[0]
+        # Points - rename depth labels to standard if a non standard one is specified
+        points_i = geopandas.read_file(points_files[0])
+        points_i[self.TYPE_LABEL] = type_labels[0]
         if z_labels is not None:
-            points = points.rename(columns={z_labels[0]: self.DEPTH_LABEL})
-        points = points[[self.DEPTH_LABEL, self.TYPE_LABEL, "geometry"]]
-        polygon = geopandas.read_file(polygon_files[0])
-        polygon[self.TYPE_LABEL] = type_labels[0]
+            points_i = points_i.rename(columns={z_labels[0]: self.DEPTH_LABEL})
+        columns_i = [self.DEPTH_LABEL, self.TYPE_LABEL, "geometry"]
+        if self.BANK_HEIGHT_LABEL in points_i.columns:
+            columns_i.append(self.BANK_HEIGHT_LABEL)
+        if self.WIDTH_LABEL in points_i.columns:
+            columns_i.append(self.WIDTH_LABEL)
+        points_i = points_i[columns_i]
+        points_list = [points_i]
+        # Polygon where the points are relevent
+        polygon_i = geopandas.read_file(polygon_files[0])
+        polygon_i[self.TYPE_LABEL] = type_labels[0]
+        polygon_list = [polygon_i]
         for i in range(1, len(points_files)):
+            # Points - rename depth labels to standard if  specified
             points_i = geopandas.read_file(points_files[i])
             points_i[self.TYPE_LABEL] = type_labels[i]
             if z_labels is not None and z_labels[i] != self.DEPTH_LABEL:
                 points_i = points_i.rename(columns={z_labels[i]: self.DEPTH_LABEL})
-            points_i = points_i[[self.DEPTH_LABEL, self.TYPE_LABEL, "geometry"]]
-            points = points.append(points_i)
+            columns_i = [self.DEPTH_LABEL, self.TYPE_LABEL, "geometry"]
+            if self.BANK_HEIGHT_LABEL in points_i.columns:
+                columns_i.append(self.BANK_HEIGHT_LABEL)
+            if self.WIDTH_LABEL in points_i.columns:
+                columns_i.append(self.WIDTH_LABEL)
+            points_i = points_i[columns_i]
+            points_list.append(points_i)
+            # Polygon where the points are relevent
             polygon_i = geopandas.read_file(polygon_files[i])
             polygon_i[self.TYPE_LABEL] = type_labels[i]
-            polygon = polygon.append(polygon_i)
+            polygon_list.append(polygon_i)
         # Set CRS, clip to size and reset index
+        points = geopandas.GeoDataFrame(
+            pandas.concat(points_list, ignore_index=True), crs=points_list[0].crs
+        )
         points = points.to_crs(self.catchment_geometry.crs["horizontal"])
+        polygon = geopandas.GeoDataFrame(
+            pandas.concat(polygon_list, ignore_index=True), crs=polygon_list[0].crs
+        )
         polygon = polygon.to_crs(self.catchment_geometry.crs["horizontal"])
         points = points.clip(polygon.buffer(0), keep_geom_type=True)
         points = points.clip(self.catchment_geometry.catchment, keep_geom_type=True)
@@ -560,9 +599,14 @@ class EstimatedBathymetryPoints:
 
         points_array = numpy.empty(
             [len(points)],
-            dtype=[("X", numpy.float64), ("Y", numpy.float64), ("Z", numpy.float64)],
+            dtype=[("X", RASTER_TYPE), ("Y", RASTER_TYPE), ("Z", RASTER_TYPE)],
         )
 
+        if len(points_array) == 0:
+            logging.warning(
+                f"No estimated waterway points with type label {type_label}"
+            )
+            return points_array
         # Extract the x, y and z values from the Shapely MultiPoints and possibly a
         # depth column
         points_array["X"] = points.apply(lambda row: row.geometry.x, axis=1).to_list()
@@ -577,6 +621,32 @@ class EstimatedBathymetryPoints:
             ).to_list()
         return points_array
 
+    def filtered_bank_height_points(self, type_label: str = None) -> numpy.ndarray:
+        """Return the points with 'Z' being the estimated bank height as a single array."""
+
+        # Filter the points by the type label
+        points = (
+            self._points
+            if type_label is None
+            else self._points[self._points["type"] == type_label]
+        )
+
+        points_array = numpy.empty(
+            [len(points)],
+            dtype=[("X", RASTER_TYPE), ("Y", RASTER_TYPE), ("Z", RASTER_TYPE)],
+        )
+
+        # Extract the x, y and z values from the Shapely MultiPoints and possibly a
+        # depth column
+        points_array["X"] = points.apply(lambda row: row.geometry.x, axis=1).to_list()
+        points_array["Y"] = points.apply(lambda row: row.geometry.y, axis=1).to_list()
+
+        # Pull out the bank heights
+        points_array["Z"] = points.apply(
+            lambda row: row[self.BANK_HEIGHT_LABEL], axis=1
+        ).to_list()
+        return points_array
+
     @property
     def points(self):
         """Return the points"""
@@ -669,7 +739,7 @@ class RiverMouthFan:
     The fan begins with the most downstream river width estimate, and ends with the
     first contour of either more than 2x the depth of the mouth. This aims to ensure
     their is a defined fan that is slightly deeper than the surrounding during
-    the transition.
+    the transition. A fan is caculated for both river bed elevation approach.
     TODO In future,it may move to defining the width as 10x the mouth width.
     TODO deal with no width at the mouth and work upstream.
 
@@ -684,6 +754,8 @@ class RiverMouthFan:
 
     FAN_ANGLE = 30
     FAN_MAX_LENGTH = 10_000
+    ELEVATION_LABEL_1 = "bed_elevation_Neal_et_al"
+    ELEVATION_LABEL_2 = "bed_elevation_Rupp_and_Smart"
 
     def __init__(
         self,
@@ -705,12 +777,10 @@ class RiverMouthFan:
     def _get_mouth_alignment(self):
         """Get the location and alignment of the river mouth."""
 
+        # Take the alignment based on the aligned channel file
         aligned_channel = geopandas.read_file(self.aligned_channel_file)
         (x, y) = aligned_channel.loc[0].geometry.xy
 
-        # Get the midpoint of the river mouth
-        mouth_point = shapely.geometry.Point([x[0], y[0]])
-
         # Calculate the normal and tangent to the channel segment at the mouth
         segment_dx = x[0] - x[1]
         segment_dy = y[0] - y[1]
@@ -720,20 +790,26 @@ class RiverMouthFan:
         )
         mouth_normal = shapely.geometry.Point([-mouth_tangent.y, mouth_tangent.x])
 
+        # Get the midpoint of the river mouth from the river bathymetry
+        river_bathymetry = geopandas.read_file(self.river_bathymetry_file)
+        mouth_point = river_bathymetry.loc[0].geometry
+
         return mouth_point, mouth_tangent, mouth_normal
 
     def _get_mouth_bathymetry(self):
         """Get the width and depth at the river mouth."""
 
         river_bathymetry = geopandas.read_file(self.river_bathymetry_file)
-        river_mouth_depth = river_bathymetry["bed_elevation_Rupp_and_Smart"].iloc[0]
-        river_mouth_width = river_bathymetry["widths"].iloc[0]
-        return river_mouth_depth, river_mouth_width
+        river_mouth_elevation_1 = river_bathymetry[self.ELEVATION_LABEL_1].iloc[0]
+        river_mouth_elevation_2 = river_bathymetry[self.ELEVATION_LABEL_2].iloc[0]
+        river_mouth_width = river_bathymetry["width"].iloc[0]
+        return river_mouth_width, [river_mouth_elevation_1, river_mouth_elevation_2]
 
     def _get_ocean_contours(
         self, river_mouth_depth, depth_sign: int = -1, depth_multiplier: int = 2
-    ):
-        """Load in the ocean contours.
+    ) -> geopandas.GeoDataFrame:
+        """Load in the ocean contours and remove those that are too close to the
+        shoreline to consider as the slope change would be too great.
 
         Parameters:
             river_mouth_depth  The depth in m of the river mouth
@@ -751,29 +827,27 @@ class RiverMouthFan:
         depth_label = self.ocean_contour_depth_label
 
         # Determine the end depth and filter the contours to include only these contours
-        end_depth = ocean_contours[depth_label][
+        ocean_contours = ocean_contours[
             ocean_contours[depth_label]
             > depth_multiplier * river_mouth_depth * depth_sign
-        ].min()
-        ocean_contours = ocean_contours[
-            ocean_contours[depth_label] == end_depth
         ].reset_index(drop=True)
 
         assert (
             len(ocean_contours) > 0
-        ), "No contours exist with a depth 2x the river mouth depth. "
+        ), f"No contours exist with a depth {depth_multiplier}x the river mouth depth. "
 
-        return ocean_contours, end_depth
+        return ocean_contours
 
     def _bathymetry(
         self,
         intersection_line: shapely.geometry.LineString,
-        river_mouth_depth: float,
+        river_mouth_elevations: list,
         end_depth: float,
         mouth_point: shapely.geometry.Point,
         mouth_tangent: shapely.geometry.Point,
     ):
-        """Calculate and return the fan bathymetry values.
+        """Calculate and return the fan bathymetry values for both river bed
+        estimation approaches.
 
         Parameters:
             intersection_line  The contour line defining the end of the fan
@@ -796,9 +870,16 @@ class RiverMouthFan:
         distance = fan_centre.intersection(intersection_line).distance(mouth_point)
 
         # Setup the fan data values
-        fan_depths = {"geometry": [], "depths": []}
+        fan_depths = {
+            "geometry": [],
+            self.ELEVATION_LABEL_1: [],
+            self.ELEVATION_LABEL_2: [],
+        }
         number_of_samples = int(distance / self.cross_section_spacing)
-        depth_increment = (-1 * end_depth - river_mouth_depth) / number_of_samples
+        depth_increments = [
+            (-1 * end_depth - river_mouth_elevations[0]) / number_of_samples,
+            (-1 * end_depth - river_mouth_elevations[1]) / number_of_samples,
+        ]
 
         # Iterate through creating fan bathymetry
         for i in range(1, number_of_samples):
@@ -812,7 +893,12 @@ class RiverMouthFan:
                     ]
                 )
             )
-            fan_depths["depths"].append(river_mouth_depth + i * depth_increment)
+            fan_depths[self.ELEVATION_LABEL_1].append(
+                river_mouth_elevations[0] + i * depth_increments[0]
+            )
+            fan_depths[self.ELEVATION_LABEL_2].append(
+                river_mouth_elevations[1] + i * depth_increments[1]
+            )
         fan_depths = geopandas.GeoDataFrame(fan_depths, crs=self.crs)
         return fan_depths
 
@@ -866,11 +952,13 @@ class RiverMouthFan:
         return fan_polygon
 
     def polygon_and_bathymetry(self):
-        """Calculate and return the fan polygon values."""
+        """Calculate and return the fan polygon values. Perform calculations for both
+        rive bed estimation approaches, so both have a smooth transition to offshore
+        bathyemtry."""
 
         # Load in river mouth alignment and bathymetry
         mouth_point, mouth_tangent, mouth_normal = self._get_mouth_alignment()
-        river_mouth_depth, river_mouth_width = self._get_mouth_bathymetry()
+        river_mouth_width, river_mouth_elevations = self._get_mouth_bathymetry()
 
         # Create maximum fan polygon
         fan_polygon = self._max_length_polygon(
@@ -881,11 +969,15 @@ class RiverMouthFan:
         )
 
         # Load in ocean depth contours
-        ocean_contours, end_depth = self._get_ocean_contours(river_mouth_depth)
+        ocean_contours = self._get_ocean_contours(max(river_mouth_elevations))
 
-        # Cycle through contours finding the nearest contour to intersect the fan
+        # Clip to fan
+        ocean_contours = ocean_contours.clip(fan_polygon)
+
+        # Cycle through contours finding the nearest contour to intersect wiith the fan
         distance = numpy.inf
         intersection_line = shapely.geometry.Point()
+        end_depth = numpy.nan
 
         for i, row in ocean_contours.iterrows():
             if row.geometry.intersects(fan_polygon):
@@ -893,6 +985,7 @@ class RiverMouthFan:
                 if intersection_line_i.distance(mouth_point) < distance:
                     distance = intersection_line_i.distance(mouth_point)
                     intersection_line = intersection_line_i
+                    end_depth = row[self.ocean_contour_depth_label]
         assert distance < numpy.inf, (
             "There must be at least one ocean "
             "contour within the max length fan polygon."
@@ -949,7 +1042,7 @@ class RiverMouthFan:
         bathymetry = self._bathymetry(
             intersection_line=intersection_line,
             end_depth=end_depth,
-            river_mouth_depth=river_mouth_depth,
+            river_mouth_elevations=river_mouth_elevations,
             mouth_point=mouth_point,
             mouth_tangent=mouth_tangent,
         )
